@@ -9,7 +9,7 @@
 """
 Myokit: The Maastricht Modeling Toolkit
 
-This module provides a gateway to the main myokit components. For example, to
+This module provides a gateway to the main Myokit components. For example, to
 load a model use myokit.load_model('filename.mmt'), create a myokit.Protocol
 and then a myokit.Simulation which you can .run() to obtain simulated results.
 """
@@ -33,38 +33,58 @@ from __future__ import absolute_import, division
 from __future__ import print_function, unicode_literals
 
 
+#
+# Configure logging
+#
+import logging  # noqa  (not at top of file)
+logging.basicConfig()
+del(logging)
+
+
 #
 # Check python version
 #
-import sys
-if sys.hexversion < 0x02070000:     # pragma: no python 3 cover
-    print('-- ERROR --')
-    print('Myokit requires Python version 2.7.0 or higher.')
-    print('Detected Python version: ')
-    print(sys.version)
-    print()
-    sys.exit(1)
-elif sys.hexversion >= 0x03000000 and sys.hexversion < 0x03040000:
-    print('-- ERROR --')
-    print('Python 3.0 to 3.3 are not supported.')
-    print()
-    sys.exit(1)
-elif sys.hexversion >= 0x03040000:
-    # Don't exit, allow testing with Python 3.4+
-    import logging      # noqa
-    logging.basicConfig()
+# Hexversion guide:
+#  0x   Hex
+#  02   PY_MAJOR_VERSION
+#  07   PY_MINOR_VERSION
+#  0F   PY_MICRO_VERSION, in hex, so 0F is 15, 10 is 16, etc.
+#  F    PY_RELEASE_LEVEL, A for alpha, B for beta, C for candidate, F for final
+#  0    PY_RELEASE_SERIAL, increments with every release
+#
+import sys  # noqa
+if sys.hexversion < 0x02070F00:     # pragma: no python 3 cover
+    import logging  # noqa
     log = logging.getLogger(__name__)
-    log.warning('Myokit support for Python3 is still experimental.')
+    log.warning(
+        'Myokit is not tested on Python 2 versions older than 2.7.15')
+    log.warning('Detected Python version: ' + sys.version)
+    del(logging, log)
+elif sys.hexversion >= 0x03000000 and sys.hexversion < 0x03050000:
+    import logging  # noqa
+    log = logging.getLogger(__name__)
+    log.warning(
+        'Myokit is not tested on Python 3 versions older than 3.5.0')
+    log.warning('Detected Python version: ' + sys.version)
     del(logging, log)
 
-# Don't expose standard libraries as part of Myokit
+
+# Exec() that works with Python 2 versions before 2.7.9
+if sys.hexversion < 0x020709F0:
+    from ._exec_old import _exec    # noqa
+else:
+    from ._exec_new import _exec    # noqa
 del(sys)
 
 
 #
 # Version information
 #
-from ._myokit_version import RELEASE, VERSION_INT, VERSION  # noqa
+from ._myokit_version import (  # noqa
+    __release__,
+    __version__,
+    __version_tuple__,
+)
 
 
 # Myokit version
@@ -73,9 +93,9 @@ def version(raw=False):
     Returns the current Myokit version.
     """
     if raw:
-        return VERSION
+        return __version__
     else:
-        t1 = ' Myokit ' + VERSION + ' '
+        t1 = ' Myokit ' + __version__ + ' '
         t2 = '_' * len(t1)
         t1 += '|/\\'
         t2 += '|  |' + '_' * 5
@@ -83,16 +103,15 @@ def version(raw=False):
 
 
 # Warn about development version
-import logging      # noqa
-logging.basicConfig()
+import logging  # noqa
 log = logging.getLogger(__name__)
-log.info('Loading Myokit version ' + VERSION)
-if not RELEASE:
+log.info('Loading Myokit version ' + __version__)
+if not __release__:
     log.warning(
         'Using development version of Myokit. This may contain untested'
         ' features and bugs. Please see http://myokit.org for the latest'
         ' stable release.')
-del(logging, log)
+del(log, logging)
 
 
 #
@@ -173,15 +192,23 @@ finally:
 # Binary data files
 DIR_DATA = os.path.join(DIR_MYOKIT, '_bin')
 
-# Windows data files
-DIR_WIN = os.path.join(DIR_MYOKIT, '_win')
-
 # C header files
 DIR_CFUNC = os.path.join(DIR_MYOKIT, '_sim')
 
-# Location of myokit user info
-DIR_USER = os.path.join(os.path.expanduser('~'), '.myokit')
-if os.path.exists(DIR_USER):
+# Location of myokit user config
+DIR_USER = os.path.join(os.path.expanduser('~'), '.config', 'myokit')
+
+# Old user config location: Move if possible
+DIR_USER_OLD = os.path.join(os.path.expanduser('~'), '.myokit')
+
+if os.path.exists(DIR_USER_OLD):    # pragma: no cover
+    if not os.path.exists(DIR_USER):
+        import shutil  # noqa
+        shutil.move(DIR_USER_OLD, DIR_USER)
+        del(shutil)
+
+# Ensure the user config directory exists and is writable
+if os.path.exists(DIR_USER):    # pragma: no cover
     if not os.path.isdir(DIR_USER):
         raise Exception(
             'File or link found in place of user directory: ' + str(DIR_USER))
@@ -259,8 +286,8 @@ SUNDIALS_LIB = []
 # Location of the Sundials (CVODE) header files (.h)
 SUNDIALS_INC = []
 
-# Sundials major version number. Defaults to 26000.
-SUNDIALS_VERSION = 26000
+# Sundials version number. Defaults to 0.
+SUNDIALS_VERSION = 0
 
 # Location of the OpenCL shared library objects (.dll or .so)
 OPENCL_LIB = []
@@ -268,35 +295,49 @@ OPENCL_LIB = []
 # Location of the OpenCL header files (.h)
 OPENCL_INC = []
 
-# Load settings
-from . import _config   # noqa
-del(_config)
-
 
 #
 # Imports
 #
 
 # Exceptions
-from ._err import ( # noqa
-    MyokitError,
-    IntegrityError, InvalidBindingError, InvalidLabelError,
-    DuplicateName, InvalidNameError, IllegalAliasError,
-    UnresolvedReferenceError, IllegalReferenceError,
-    UnusedVariableError, CyclicalDependencyError,
-    MissingRhsError, MissingTimeVariableError,
-    NonLiteralValueError, NumericalError,
-    IncompatibleUnitError, InvalidMetaDataNameError,
-    DuplicateFunctionName, DuplicateFunctionArgument,
-    InvalidFunction,
-    ParseError, SectionNotFoundError,
-    ProtocolParseError, ProtocolEventError,
-    SimultaneousProtocolEventError,
-    SimulationError, FindNanError, SimulationCancelledError,
-    InvalidDataLogError, DataLogReadError, DataBlockReadError,
-    GenerationError, CompilationError,
-    ImportError, ExportError,
+from ._err import (  # noqa
+    CompilationError,
+    CyclicalDependencyError,
+    DataBlockReadError,
+    DataLogReadError,
+    DuplicateFunctionArgument,
+    DuplicateFunctionName,
+    DuplicateName,
+    ExportError,
+    FindNanError,
+    GenerationError,
+    IllegalAliasError,
+    IllegalReferenceError,
+    ImportError,
     IncompatibleModelError,
+    IncompatibleUnitError,
+    IntegrityError,
+    InvalidBindingError,
+    InvalidDataLogError,
+    InvalidFunction,
+    InvalidLabelError,
+    InvalidMetaDataNameError,
+    InvalidNameError,
+    MissingRhsError,
+    MissingTimeVariableError,
+    MyokitError,
+    NonLiteralValueError,
+    NumericalError,
+    ParseError,
+    ProtocolEventError,
+    ProtocolParseError,
+    SectionNotFoundError,
+    SimulationCancelledError,
+    SimulationError,
+    SimultaneousProtocolEventError,
+    UnresolvedReferenceError,
+    UnusedVariableError,
 )
 
 # Check if all errors imported
@@ -318,49 +359,109 @@ del(_err)
 
 # Model api
 from ._model_api import ( # noqa
-    ModelPart, Model, Component, Variable, check_name,
-    Equation, EquationList, UserFunction,
+    check_name,
+    Component,
+    Equation,
+    EquationList,
+    Model,
+    ModelPart,
+    UserFunction,
+    Variable,
 )
 
 # Expressions and units
-from ._expressions import ( # noqa
-    Expression, LhsExpression, Derivative, Name, Number,
-    PrefixExpression, PrefixPlus, PrefixMinus,
-    InfixExpression, Plus, Minus, Multiply, Divide,
-    Quotient, Remainder, Power,
+from ._expressions import (  # noqa
+    Abs,
+    ACos,
+    And,
+    ASin,
+    ATan,
+    Ceil,
+    Condition,
+    Cos,
+    Derivative,
+    Divide,
+    Equal,
+    Exp,
+    Expression,
+    Floor,
     Function,
-    Sqrt, Exp, Log, Log10, Sin, Cos, Tan, ASin, ACos, ATan, Floor, Ceil, Abs,
-    Condition, PrefixCondition, InfixCondition, If, Piecewise,
-    Not, And, Or, Equal, NotEqual, More, Less, MoreEqual, LessEqual,
+    If,
+    InfixCondition,
+    InfixExpression,
+    Less,
+    LessEqual,
+    LhsExpression,
+    Log,
+    Log10,
+    Minus,
+    More,
+    MoreEqual,
+    Multiply,
+    Name,
+    Number,
+    Not,
+    NotEqual,
+    Or,
+    Piecewise,
+    Plus,
+    Power,
+    PrefixCondition,
+    PrefixExpression,
+    PrefixMinus,
+    PrefixPlus,
+    Quantity,
+    Quotient,
+    Remainder,
+    Sin,
+    Sqrt,
+    Tan,
     UnsupportedFunction,
-    Unit, Quantity,
+    Unit,
+
 )
 
 # Pacing protocol
-from ._protocol import ( # noqa
-    Protocol, ProtocolEvent, PacingSystem,
+from ._protocol import (  # noqa
+    PacingSystem,
+    Protocol,
+    ProtocolEvent,
 )
 
 # Parser functions
-from ._parsing import ( # noqa
+from ._parsing import (  # noqa
+    format_parse_error,
     KEYWORDS,
-    parse, split, format_parse_error,
-    parse_model, parse_protocol, parse_state,
-    parse_unit_string as parse_unit,
-    #parse_number_string as parse_number,
+    parse,
     parse_expression_string as parse_expression,
+    parse_model,
+    #parse_number_string as parse_number,
+    parse_protocol,
+    parse_state,
+    parse_unit_string as parse_unit,
+    split,
     strip_expression_units,
 )
 
 # Auxillary functions
-from ._aux import ( # noqa
+from ._aux import (  # noqa
     # Global date and time formats
-    date, time,
+    date,
+    time,
 
     # Reading, writing
-    load, load_model, load_protocol, load_script,
-    save, save_model, save_protocol, save_script,
-    load_state, save_state, load_state_bin, save_state_bin,
+    load,
+    load_model,
+    load_protocol,
+    load_script,
+    save,
+    save_model,
+    save_protocol,
+    save_script,
+    load_state,
+    save_state,
+    load_state_bin,
+    save_state_bin,
 
     # Running scripts
     run,
@@ -369,13 +470,15 @@ from ._aux import ( # noqa
     step,
 
     # Output masking
-    PyCapture, SubCapture,
+    PyCapture,
+    SubCapture,
 
     # Sorting
-    natural_sort_key,
+    _natural_sort_key,
 
-    # Dyanmic generation of Python/Numpy expressions
-    python_writer, numpy_writer,
+    # Dyanmic generation of Python/NumPy expressions
+    python_writer,
+    numpy_writer,
 
     # Model comparison
     ModelComparison,
@@ -384,45 +487,67 @@ from ._aux import ( # noqa
     Benchmarker,
 
     # Misc
-    lvsd, format_path, strfloat, format_float_dict,
+    _lvsd,
+    format_path,
+    strfloat,
+    format_float_dict,
 
     # Snapshot creation for replicability
     pack_snapshot,
 )
 
+# System information
+from ._system import (      # noqa
+    system,
+)
 
 # Progress reporting
 from ._progress import (    # noqa
-    ProgressReporter, ProgressPrinter,
+    ProgressPrinter,
+    ProgressReporter,
+    Timeout,
 )
 
 # Data logging
 from ._datalog import (     # noqa
-    DataLog, LoggedVariableInfo, dimco, split_key, prepare_log,
+    DataLog,
+    _dimco,
+    LoggedVariableInfo,
+    prepare_log,
+    split_key,
 )
 from ._datablock import (   # noqa
-    DataBlock1d, DataBlock2d, ColorMap,
+    ColorMap,
+    DataBlock1d,
+    DataBlock2d,
 )
 
 
 # Simulations
-from ._sim import ( # noqa
-    CModule, CppModule,
+from ._sim import (  # noqa
+    CModule,
+    CppModule,
 )
-from ._sim.cvode import Simulation          # noqa
+from ._sim.compiler import (  # noqa
+    Compiler,
+)
+from ._sim.sundials import (  # noqa
+    Sundials,
+)
+from ._sim.opencl import (  # noqa
+    OpenCL,
+    OpenCLDeviceInfo,
+    OpenCLInfo,
+    OpenCLPlatformInfo,
+)
+from ._sim.cvodesim import Simulation       # noqa
 from ._sim.cable import Simulation1d        # noqa
 from ._sim.rhs import RhsBenchmarker        # noqa
 from ._sim.icsim import ICSimulation        # noqa
 from ._sim.psim import PSimulation          # noqa
 from ._sim.jacobian import JacobianTracer, JacobianCalculator   # noqa
-# from ._sim.openmp import SimulationOpenMP
-from ._sim.opencl import ( # noqa
-    OpenCL,
-    OpenCLInfo,
-    OpenCLPlatformInfo,
-    OpenCLDeviceInfo,
-)
-from ._sim.openclsim import SimulationOpenCL    # noqa
+#from ._sim.openmp import SimulationOpenMP                       # noqa
+from ._sim.openclsim import SimulationOpenCL                    # noqa
 from ._sim.fiber_tissue import FiberTissueSimulation            # noqa
 
 # Import whole modules
@@ -442,6 +567,13 @@ from . import ( # noqa
 _Simulation_progress = None
 
 
+#
+# Load settings
+#
+from . import _config   # noqa
+del(_config)
+
+
 #
 # Default mmt file parts
 #
