@@ -62,13 +62,13 @@ def time():
     return t.strftime(myokit.TIME_FORMAT)
 
 
-def natural_sort_key(s):
+def _natural_sort_key(s):
     """
     Natural sort key, from: http://stackoverflow.com/questions/4836710/
 
     Usage example::
 
-        values.sort(key=lambda x: myokit.natural_sort_key(x))
+        values.sort(key=lambda x: myokit._natural_sort_key(x))
 
     """
     return [
@@ -528,6 +528,8 @@ def load_state_bin(filename):
         code = parts[1]
         if code not in ['d', 'f']:  # pragma: no cover
             raise Exception('Invalid state file format [40].')
+        # Convert code to str for Python 2.7.10 (see #225)
+        code = str(code)
 
         size = int(parts[2])
         if size < 0:    # pragma: no cover
@@ -547,7 +549,7 @@ def load_state_bin(filename):
     return list(ar)
 
 
-def lvsd(s1, s2):
+def _lvsd(s1, s2):
     """
     Calculates a Levenshtein distance, as found on wikibooks
 
@@ -556,7 +558,7 @@ def lvsd(s1, s2):
     :returns: int The distance between s1 and s2
     """
     if len(s1) < len(s2):
-        return lvsd(s2, s1)
+        return _lvsd(s2, s1)
 
     if not s1:
         return len(s2)
@@ -814,8 +816,8 @@ def numpy_writer():
     """
     global _numpywriter_
     if _numpywriter_ is None:
-        from .formats.python import NumpyExpressionWriter
-        _numpywriter_ = NumpyExpressionWriter()
+        from .formats.python import NumPyExpressionWriter
+        _numpywriter_ = NumPyExpressionWriter()
 
         def name(x):
             u = x.var()
@@ -853,7 +855,7 @@ def pack_snapshot(path, overwrite=True):
                 raise IOError('File already exists at given path.')
         else:
             path = os.path.join(
-                path, 'myokit_' + myokit.VERSION + '_snapshot.zip')
+                path, 'myokit_' + myokit.__version__ + '_snapshot.zip')
             if os.path.isfile(path) and not overwrite:
                 raise IOError('File already exists as given path.')
 
@@ -1018,7 +1020,7 @@ def run(model, protocol, script, stdout=None, stderr=None, progress=None):
                     'get_model': get_model,
                     'get_protocol': get_protocol,
                 }
-                exec(self.script, environment)
+                myokit._exec(self.script, environment)
             finally:
                 if oldstdout is not None:
                     sys.stdout = oldstdout
@@ -1163,7 +1165,8 @@ def save_state_bin(filename, state, precision=myokit.DOUBLE_PRECISION):
             'This method requires the `zlib` module to be installed.')
 
     # Data type
-    code = 'd' if precision == myokit.DOUBLE_PRECISION else 'f'
+    # Convert code to str for Python 2.7.10 (see #225)
+    code = str('d' if precision == myokit.DOUBLE_PRECISION else 'f')
 
     # Create array, ensure it's little-endian
     ar = array.array(code, state)
@@ -1301,10 +1304,14 @@ def step(model, initial=None, reference=None, ignore_errors=False):
     return '\n'.join(log)
 
 
-def strfloat(number):
+def strfloat(number, full=False):
     """
-    Turns the given number into a string, without losing accuracy.
+    Turns the given number into a string.
     """
+    # Force full precision output
+    if full:
+        return myokit.SFDOUBLE.format(float(number))
+
     # Pass through strings
     if isinstance(number, str):
         return number
