@@ -246,7 +246,7 @@ class DataBlock1d(object):
         try:
             # Newer numpy wants rcond=None
             cv = np.linalg.lstsq(A, x, rcond=None)[0][0]
-        except TypeError:
+        except TypeError:   # pragma: no cover
             cv = np.linalg.lstsq(A, x)[0][0]
 
         # Return
@@ -268,9 +268,7 @@ class DataBlock1d(object):
         infos = log.variable_info()
 
         # Check time variable
-        try:
-            info = infos[time]
-        except KeyError:
+        if time not in infos:
             # Already checked time variable exists, so if not found now must
             # be multi-dimensional
             raise ValueError('Time variable must be 0-dimensional.')
@@ -475,10 +473,11 @@ class DataBlock1d(object):
                 'd': len(array.array('d', [1]).tobytes()),
                 'f': len(array.array('f', [1]).tobytes()),
             }
-        except AttributeError:  # pragma: no python 3 cover
+        except (AttributeError, TypeError):  # pragma: no python 3 cover
+            # List dtype as str for Python 2.7.10 (see #225)
             dsize = {
-                'd': len(array.array('d', [1]).tostring()),
-                'f': len(array.array('f', [1]).tostring()),
+                b'd': len(array.array(b'd', [1]).tostring()),
+                b'f': len(array.array(b'f', [1]).tostring()),
             }
 
         # Read data from file
@@ -537,10 +536,12 @@ class DataBlock1d(object):
             nt = int(next(head))
             nx = int(next(head))
             dtype = next(head)[1:-1]
+            # Convert dtype to str for Python 2.7.10 (see #225)
+            dtype = str(dtype)
             if dtype not in dsize:
                 raise myokit.DataBlockReadError(
                     'Unable to read DataBlock1d: Unrecognized data type "'
-                    + str(dtype) + '".')
+                    + dtype + '".')
             names_0d = []
             names_1d = []
             name = next(head)
@@ -663,7 +664,8 @@ class DataBlock1d(object):
                 'This method requires the ``zlib`` module to be installed.')
 
         # Data type
-        dtype = 'd'     # Only supporting doubles right now
+        # Create dtype as str for Python 2.7.10 (see #225)
+        dtype = str('d')     # Only supporting doubles right now
 
         # Create header
         head_str = []
@@ -1032,9 +1034,7 @@ class DataBlock2d(object):
         infos = log.variable_info()
 
         # Check time variable
-        try:
-            info = infos[time]
-        except KeyError:
+        if time not in infos:
             # Already checked time variable exists, so if not found now must
             # be multi-dimensional
             raise ValueError('Time variable must be 0-dimensional.')
@@ -1211,10 +1211,11 @@ class DataBlock2d(object):
                 'd': len(array.array('d', [1]).tobytes()),
                 'f': len(array.array('f', [1]).tobytes()),
             }
-        except AttributeError:  # pragma: no python 3 cover
+        except (AttributeError, TypeError):  # pragma: no python 3 cover
+            # List dtype as str for Python 2.7.10 (see #225)
             dsize = {
-                'd': len(array.array('d', [1]).tostring()),
-                'f': len(array.array('f', [1]).tostring()),
+                b'd': len(array.array(b'd', [1]).tostring()),
+                b'f': len(array.array(b'f', [1]).tostring()),
             }
 
         # Read data from file
@@ -1234,7 +1235,7 @@ class DataBlock2d(object):
             except ValueError:
                 # Attempt reading as DataBlock1d
                 try:
-                    head = names.index('header_block1d.txt')
+                    names.index('header_block1d.txt')
                 except ValueError:
                     raise myokit.DataBlockReadError(
                         'Invalid DataBlock2d file format: Header not found.')
@@ -1283,6 +1284,8 @@ class DataBlock2d(object):
             ny = int(next(head))
             nx = int(next(head))
 
+            # Get dtype
+            # Convert dtype to str for Python 2.7.10 (see #225)
             dtype = str(next(head))[1:-1]
             if dtype not in dsize:
                 raise myokit.DataBlockReadError(
@@ -1417,7 +1420,8 @@ class DataBlock2d(object):
                 'This method requires the ``zlib`` module to be installed.')
 
         # Data type
-        dtype = 'd'     # Only supporting doubles right now
+        # Create dtype as str for Python 2.7.10 (see #225)
+        dtype = str('d')  # Only supporting doubles right now
 
         # Create header
         head_str = []
@@ -1605,7 +1609,7 @@ class ColorMap(object):
     """
     _colormaps = {}
 
-    def __call__(floats, lower=None, upper=None, alpha=True, rgb=None):
+    def __call__(self, floats, lower=None, upper=None, alpha=True, rgb=None):
         raise NotImplementedError
 
     @staticmethod
