@@ -487,10 +487,11 @@ class DataLog(OrderedDict):
                 'd': len(array.array('d', [1]).tobytes()),
                 'f': len(array.array('f', [1]).tobytes()),
             }
-        except AttributeError:  # pragma: no python 3 cover
+        except (AttributeError, TypeError):  # pragma: no python 3 cover
+            # List dtype as str for Python 2.7.10 (see #225)
             dsize = {
-                'd': len(array.array('d', [1]).tostring()),
-                'f': len(array.array('f', [1]).tostring()),
+                b'd': len(array.array(b'd', [1]).tostring()),
+                b'f': len(array.array(b'f', [1]).tostring()),
             }
 
         # Read data
@@ -527,7 +528,7 @@ class DataLog(OrderedDict):
         head = iter(head.splitlines())
         n = int(next(head))
         data_size = int(next(head))
-        data_type = next(head)
+        data_type = str(next(head))  # Cast to str for Python 2.7.10 (see #225)
         time = next(head)
         if time:
             # Note, this field doesn't have to be present in the log!
@@ -602,7 +603,8 @@ class DataLog(OrderedDict):
         filename = os.path.expanduser(filename)
 
         # Typecode dependent on precision
-        typecode = 'd' if precision == myokit.DOUBLE_PRECISION else 'f'
+        # Typecode must be str for Python 2.7.10 (see #225)
+        typecode = str('d' if precision == myokit.DOUBLE_PRECISION else 'f')
 
         # Error raising function
         def e(line, char, msg):
@@ -888,7 +890,8 @@ class DataLog(OrderedDict):
                 'This method requires the `zlib` module to be installed.')
 
         # Data type
-        dtype = 'd' if precision == myokit.DOUBLE_PRECISION else 'f'
+        # dtype must be str for Python 2.7.10 (see #225)
+        dtype = str('d' if precision == myokit.DOUBLE_PRECISION else 'f')
 
         # Create data strings
         head_str = []
@@ -1139,7 +1142,7 @@ class DataLog(OrderedDict):
             # Select sections of log and append
             for k, v in self.items():
                 d = self[k][imin:imax]
-                # Numpy? Then copy data
+                # NumPy? Then copy data
                 if isinstance(d, np.ndarray):
                     d = np.array(d, copy=True, dtype=float)
                 log[k] = d
@@ -1158,7 +1161,7 @@ class DataLog(OrderedDict):
         # Select sections of log and append
         for k, v in self.items():
             d = self[k][imin:imax]
-            # Numpy? Then copy data
+            # NumPy? Then copy data
             if isinstance(d, np.ndarray):
                 d = np.array(d, copy=True, dtype=float)
             log[k] = d
@@ -1516,7 +1519,6 @@ def prepare_log(
     Returns a :class:`DataLog` for simulation classes based on a ``log``
     argument passed in by the user. The model the simulations will be based on
     should be passed in as ``model``.
-    as
 
     The ``log`` argument can take on one of four forms:
 
@@ -1592,7 +1594,7 @@ def prepare_log(
     if ndims == 0:
         dcombos = ['']
     else:
-        dcombos = ['.'.join([str(y) for y in x]) + '.' for x in dimco(*dims)]
+        dcombos = ['.'.join([str(y) for y in x]) + '.' for x in _dimco(*dims)]
 
     # Check given list of global variables
     if global_vars is None:
@@ -1611,16 +1613,23 @@ def prepare_log(
     # Function to check if variable is allowed (doesn't handle derivatives)
     def check_if_allowed_class(var):
         if var.is_constant():
-            raise ValueError('This log does not support constants.')
+            raise ValueError(
+                'This log does not support constants, got <'
+                + str(var) + '>.')
         elif var.is_state():
             if not myokit.LOG_STATE & allowed_classes:
-                raise ValueError('This log does not support state variables.')
+                raise ValueError(
+                    'This log does not support state variables, got <'
+                    + str(var) + '>.')
         elif var.is_bound():
             if not myokit.LOG_BOUND & allowed_classes:
-                raise ValueError('This log does not support bound variables.')
+                raise ValueError(
+                    'This log does not support bound variables, got <'
+                    + str(var) + '>.')
         elif not myokit.LOG_INTER & allowed_classes:
             raise ValueError(
-                'This log does not support intermediary variables.')
+                'This log does not support intermediary variables, got <'
+                + str(var) + '>.')
 
     #
     # First option, no log argument given, use the "if_empty" option
@@ -1762,7 +1771,9 @@ def prepare_log(
             # Check if in class of allowed variables
             if deriv:
                 if not myokit.LOG_DERIV & allowed_classes:
-                    raise ValueError('This log does not support derivatives.')
+                    raise ValueError(
+                        'This log does not support derivatives, got <'
+                        + key + '>.')
                 if not var.is_state():
                     raise ValueError(
                         'Cannot log time derivative of non-state <'
@@ -1856,7 +1867,9 @@ def prepare_log(
             if deriv:
                 if not myokit.LOG_DERIV & allowed_classes:
                     raise ValueError(
-                        'This log does not support derivatives.')
+                        'This log does not support derivatives, got <'
+                        + key + '>.')
+
                 if not var.is_state():
                     raise ValueError(
                         'Cannot log time derivative of non-state <'
@@ -1901,7 +1914,7 @@ def prepare_log(
     return log
 
 
-def dimco(*dims):
+def _dimco(*dims):
     """
     Generates all the combinations of a certain set of integer dimensions. For
     example given ``dims=(2, 3)`` it returns::
