@@ -11,6 +11,7 @@ from __future__ import print_function, unicode_literals
 
 import os
 import myokit
+import platform
 
 # OpenCLSim keywords
 from .openclsim import KEYWORDS
@@ -53,10 +54,12 @@ class FiberTissueSimulation(myokit.CModule):
         The fiber-cell to tissue-cell conductance at the junction (a scalar).
     ``dt``
         The time step to use in the forward-Euler integration scheme.
-    ``double_precision``
-        Set this to True if your OpenCL device supports double precision. This
-        will greatly reduce the chance of a divide-by-zero or other numerical
-        error introducing NaNs into the simulation.
+    ``precision``
+        Can be set to ``myokit.SINGLE_PRECISION`` (default) or
+        ``myokit.DOUBLE_PRECISION`` if the used device supports it.
+    ``native_maths``
+        On some devices, selected functions (e.g. ``exp``) can be made to run
+        faster (but possibly less accurately) by setting ``native_maths=True``.
 
     The simulation provides the following inputs variables can bind to:
 
@@ -126,7 +129,7 @@ class FiberTissueSimulation(myokit.CModule):
             self, fiber_model, tissue_model, protocol=None,
             ncells_fiber=(128, 2), ncells_tissue=(128, 128), nx_paced=5,
             g_fiber=(9, 6), g_tissue=(9, 6), g_fiber_tissue=9,
-            dt=0.005, double_precision=False):
+            dt=0.005, precision=myokit.SINGLE_PRECISION, native_maths=False):
         super(FiberTissueSimulation, self).__init__()
 
         # List of globally logged inputs
@@ -153,13 +156,16 @@ class FiberTissueSimulation(myokit.CModule):
                 'This simulation requires models without interdependent'
                 ' components. Please restructure the tissue model and re-run.'
                 ' Cycles:\n' + cycles)
+
         # Clone models, store
         fiber_model = fiber_model.clone()
         tissue_model = tissue_model.clone()
         self._modelf = fiber_model
         self._modelt = tissue_model
+
         # Set protocol
         self.set_protocol(protocol)
+
         # Check dimensionality, number of cells
         msg = 'The fiber size must be a tuple (nx, ny).'
         try:
@@ -216,12 +222,13 @@ class FiberTissueSimulation(myokit.CModule):
             raise ValueError('The step size must be greater than zero.')
         self._step_size = dt
 
-        # Check precision, set native math flag
-        self._precision = myokit.DOUBLE_PRECISION if double_precision else \
-            myokit.SINGLE_PRECISION
+        # Set precision
+        if precision not in (myokit.SINGLE_PRECISION, myokit.DOUBLE_PRECISION):
+            raise ValueError('Only single and double precision are supported.')
+        self._precision = precision
 
-        # Always use native maths
-        self._native_math = True
+        # Set native maths
+        self._native_math = bool(native_maths)
 
         # Set remaining properties
         self._time = 0
@@ -358,8 +365,15 @@ class FiberTissueSimulation(myokit.CModule):
             )
             return
 
+        # Define libraries
+        libs = []
+        plat = platform.system()
+        if plat != 'Darwin':     # pragma: no osx cover
+            libs.append('OpenCL')
+        if plat != 'Windows':    # pragma: no windows cover
+            libs.append('m')
+
         # Create extension
-        libs = ['OpenCL']
         libd = list(myokit.OPENCL_LIB)
         incd = list(myokit.OPENCL_INC)
         incd.append(myokit.DIR_CFUNC)
@@ -466,11 +480,11 @@ class FiberTissueSimulation(myokit.CModule):
                         if ifirst == 0:
                             break
                 elif not np.isfinite(ar[ifirst - 1]):
-                        # Earlier NaN found
-                        kfirst = key
-                        ifirst = bisect(ar, 0, ifirst)
-                        if ifirst == 0:
-                            break
+                    # Earlier NaN found
+                    kfirst = key
+                    ifirst = bisect(ar, 0, ifirst)
+                    if ifirst == 0:
+                        break
             return ifirst, kfirst
 
         # Get the name of a time variable in the fiber model
@@ -498,11 +512,11 @@ class FiberTissueSimulation(myokit.CModule):
             # Get last logged states before error
             statef = []
             statet = []
-            for dims in myokit.dimco(*self._ncellsf):
+            for dims in myokit._dimco(*self._ncellsf):
                 pre = '.'.join([str(x) for x in dims]) + '.'
                 for s in self._modelf.states():
                     statef.append(_logf[pre + s.qname()][istart])
-            for dims in myokit.dimco(*self._ncellst):
+            for dims in myokit._dimco(*self._ncellst):
                 pre = '.'.join([str(x) for x in dims]) + '.'
                 for s in self._modelt.states():
                     statet.append(_logt[pre + s.qname()][istart])
@@ -769,6 +783,7 @@ class FiberTissueSimulation(myokit.CModule):
             'native_math': self._native_math,
             'diffusion': True,
             'fields': [],
+            'rl_states': {},
         }
         args['model'] = self._modelf
         args['vmvar'] = self._vmf
