@@ -49,10 +49,18 @@ class OpenCL(myokit.CModule):
         mname = 'myokit_opencl_info_' + str(OpenCL._index)
         fname = os.path.join(myokit.DIR_CFUNC, SOURCE_FILE)
         args = {'module_name': mname}
-        libs = ['OpenCL']
+
+        # Define libraries
+        libs = []
+        import platform
+        if platform.system() != 'Darwin':     # pragma: no osx cover
+            libs.append('OpenCL')
+
+        # Add include / linker paths
         libd = list(myokit.OPENCL_LIB)
         incd = list(myokit.OPENCL_INC)
         incd.append(myokit.DIR_CFUNC)
+
         try:
             OpenCL._message = None
             OpenCL._instance = self._compile(
@@ -97,10 +105,12 @@ class OpenCL(myokit.CModule):
         with the platform/device name, or ``None`` if no preference was set.
         """
         platform, device = OpenCL.load_selection_bytes()
-        if platform:
+        if platform is not None:
             platform = platform.decode('ascii')
-        if device:
-            device = platform.decode('device')
+        if device is not None:
+            device = device.decode('ascii')
+
+        return platform, device
 
     @staticmethod
     def load_selection_bytes():
@@ -133,9 +143,9 @@ class OpenCL(myokit.CModule):
             device = get('selection', 'device')
 
         # Ensure platform and device are ascii compatible byte strings, or None
-        if platform:
+        if platform is not None:
             platform = platform.encode('ascii')
-        if device:
+        if device is not None:
             device = device.encode('ascii')
 
         return platform, device
