@@ -41,6 +41,7 @@
 
 #include <Python.h>
 #include <stdio.h>
+#include <float.h>
 
 /*
  * Event-based pacing error flags
@@ -48,6 +49,7 @@
 typedef int ESys_Flag;
 #define ESys_OK                              0
 #define ESys_OUT_OF_MEMORY                  -1
+#define ESys_PYTHON_INTERRUPT               -2
 // General
 #define ESys_INVALID_SYSTEM                 -10
 #define ESys_POPULATED_SYSTEM               -11
@@ -64,6 +66,22 @@ typedef int ESys_Flag;
 // ESys_ScheduleEvent
 #define ESys_SIMULTANEOUS_EVENT             -50
 
+/*
+ * Calculates the absolute values of a and b and returns whatever's largest.
+ */
+#define ESys_scale(a, b) (fabs(a) > fabs(b) ? fabs(a) : fabs(b))
+
+/*
+ * Tests if `a` and `b` withing float rounding-error distance of each other
+ */
+#define ESys_eq(a, b) ((a == b) || (fabs(a - b) / ESys_scale(a, b) < DBL_EPSILON))
+
+/*
+ * Tests if `a > b` or if `a` and `b` are within float rounding-error distance
+ * of each other.
+ */
+#define ESys_geq(a, b) ((a >= b) || ESys_eq(a, b))
+
 /*
  * Sets a python exception based on an event-based pacing error flag.
  *
@@ -80,6 +98,9 @@ ESys_SetPyErr(ESys_Flag flag)
     case ESys_OUT_OF_MEMORY:
         PyErr_SetString(PyExc_Exception, "E-Pacing error: Memory allocation failed.");
         break;
+    case ESys_PYTHON_INTERRUPT:
+        PyErr_SetString(PyExc_Exception, "E-Pacing error: Process interrupted by Python signal.");
+        break;
     // General
     case ESys_INVALID_SYSTEM:
         PyErr_SetString(PyExc_Exception, "E-Pacing error: Invalid pacing system provided.");
@@ -203,7 +224,7 @@ ESys_ScheduleEvent(Event head, Event add, ESys_Flag* flag)
  */
 struct ESys_Mem {
     Py_ssize_t n_events;   // The number of events in this system
-    double time;    // The current time    
+    double time;    // The current time
     Event events;   // The events, stored as an array
     Event head;     // The head of the event queue
     Event fire;     // The currently active event
@@ -452,39 +473,43 @@ ESys_Populate(ESys sys, PyObject* protocol)
  *  sys      : The pacing system to advance.
  *  new_time : The time to increment the system to. Must be more than or equal
  *             to the current pacing system time.
- *  max_time : The maximum time to advance to.
  *
  * Returns a pacing error flag.
  */
 ESys_Flag
-ESys_AdvanceTime(ESys sys, double new_time, double max_time)
+ESys_AdvanceTime(ESys sys, double new_time)
 {
-    ESys_Flag flag;     // Need to be declared here for C89 Visual C
+    ESys_Flag flag;     /* Need to be declared here for C89 Visual C */
     if(sys == 0) return ESys_INVALID_SYSTEM;
     if(sys->n_events < 0) return ESys_UNPOPULATED_SYSTEM;
-    if(sys->time > new_time) return ESys_NEGATIVE_TIME_INCREMENT;
 
-    // Update internal time
+    /* Check new_time isn't in the past */
+    if(new_time < sys->time) return ESys_NEGATIVE_TIME_INCREMENT;
+
+    /* Update internal time */
     sys->time = new_time;
-    if (new_time > max_time) max_time = new_time;
 
-    // Advance
-    while (sys->tnext <= sys->time && sys->tnext < max_time) {
-        // Active event finished
-        if (sys->fire != 0 && sys->tnext >= sys->tdown) {
+    /* Advance */
+    while (ESys_geq(sys->time, sys->tnext)) {
+
+        /* Active event finished */
+        if (sys->fire != 0 && ESys_geq(sys->tnext, sys->tdown)) {
             sys->fire = 0;
             sys->level = 0;
         }
-        // New event starting
-        if (sys->head != 0 && sys->tnext >= sys->head->start) {
+
+        /* New event starting */
+        if (sys->head != 0 && ESys_geq(sys->tnext, sys->head->start)) {
             sys->fire = sys->head;
             sys->head = sys->head->next;
             sys->tdown = sys->fire->start + sys->fire->duration;
             sys->level = sys->fire->level;
-            // Reschedule recurring event
+
+            /* Reschedule recurring event */
             if (sys->fire->period > 0) {
                 if (sys->fire->multiplier != 1) {
                     if (sys->fire->multiplier > 1) sys->fire->multiplier--;
+                    /* TODO: Replace by int-multiplication */
                     sys->fire->start += sys->fire->period;
                     sys->head = ESys_ScheduleEvent(sys->head, sys->fire, &flag);
                     if (flag != ESys_OK) { return flag; }
@@ -492,14 +517,30 @@ ESys_AdvanceTime(ESys sys, double new_time, double max_time)
                     sys->fire->period = 0;
                 }
             }
+
+            /*
+             * Check if tdown is indistinguishable from the next event start
+             * If so, then set tdown (which is always calculated) to the next
+             * event start (which may be user-specified).
+             */
+            if (sys->head != 0 && ESys_eq(sys->head->start, sys->tdown)) {
+                sys->tdown = sys->head->start;
+            }
         }
-        // Set next stopping time
-        sys->tnext = max_time;
+
+        /* Set next stopping time */
+        sys->tnext = HUGE_VAL;
         if (sys->fire != 0 && sys->tnext > sys->tdown)
             sys->tnext = sys->tdown;
         if (sys->head != 0 && sys->tnext > sys->head->start)
             sys->tnext = sys->head->start;
+
+        /* Allow interrupting if something goes wrong */
+        if (PyErr_CheckSignals() != 0) {
+            return ESys_PYTHON_INTERRUPT;
+        }
     }
+
     return ESys_OK;
 }
 
