@@ -154,7 +154,7 @@ import os
 
 
 # Encoding for text parts of files
-_ENC = 'ascii'
+_ENC = 'latin-1'
 
 
 class AbfFile(object):
@@ -201,6 +201,8 @@ class AbfFile(object):
     """
     def __init__(self, filepath, is_protocol_file=None):
         # The path to the file and its basename
+        # Ensure it's a `str` for Python2/3 compatibility
+        filepath = str(filepath)
         self._filepath = os.path.abspath(filepath)
         self._filename = os.path.basename(filepath)
 
@@ -238,7 +240,8 @@ class AbfFile(object):
             self._rate = 1e6 / self._header['protocol']['fADCSequenceInterval']
             self._mode = self._header['protocol']['nOperationMode']
         if self._mode not in acquisition_modes:
-            raise NotImplementedError('Unknown mode: ' + str(mode))
+            raise NotImplementedError(
+                'Unknown acquisition mode: ' + str(mode))
 
         # Conversion factors for integer data in the channels
         self._adc_factors = None
@@ -249,7 +252,10 @@ class AbfFile(object):
         try:
             self._protocol = self._read_protocol()
         except Exception:   # pragma: no cover
-            # If this happens, we need to update the code!
+            # This is not something we _want_ to happen, so if we have test
+            # cases that trigger this error they should be resolved. At the
+            # same time, if it happens to a user we want it to "sort-of work"
+            # (an experimental rather than a production setting)
             logging.basicConfig()
             log = logging.getLogger(__name__)
             log.warning('Unable to read protocol from ' + self._filepath)
@@ -266,26 +272,43 @@ class AbfFile(object):
         """
         Returns the number of channels in this file's sweeps.
         """
+        if len(self._sweeps) == 0:  # pragma: no cover
+            return 0
         return len(self._sweeps[0])
 
-    def extract_channel(self, channel=0):
+    def extract_channel(self, channel=0, join=False):
         """
-        Extracts the given data channel and returns it as a list containg:
+        Extracts a selected data ``channel`` and returns its data in a tuple
+        containing::
 
-             A time vector
-             The first sweep
-             The second sweep
+             A numpy array representing time
+             A numpy array representing the first sweep
+             A numpy array representing the second sweep
              ...
 
-        Each vector is returned as a numpy array.
+        An optional argument ``join=True`` can be set to join all sweeps
+        together and return just two arrays, one for time and one for data.
+
+        If no data is available, ``None`` is returned.
         """
-        data = []
         if len(self._sweeps) == 0:  # pragma: no cover
-            return data
+            return None
+
+        # Join all sweeps
+        if join:
+            time, data = [], []
+            t = np.array(self._sweeps[0][channel].times())
+            for i, sweep in enumerate(self._sweeps):
+                time.append(t + i * self._sweepStartToStart)
+                data.append(np.array(sweep[channel].values()))
+            return (np.concatenate(time), np.concatenate(data))
+
+        # Standard reading
+        data = []
         data.append(np.array(self._sweeps[0][channel].times()))
         for sweep in self._sweeps:
             data.append(np.array(sweep[channel].values()))
-        return data
+        return tuple(data)
 
     def extract_channel_as_myokit_log(self, channel=0):
         """
@@ -345,16 +368,16 @@ class AbfFile(object):
         if self._numberOfTrials:
             out.append(
                 'Protocol set for ' + str(self._numberOfTrials)
-                + ' trials, measuring ' + str(self._trialStartToStart)
-                + 's start-to-start.')
+                + ' trials, spaced ' + str(self._trialStartToStart)
+                + 's apart.')
             out.append(
                 '    with ' + str(self._runsPerTrial)
-                + ' runs per trial, measuring ' + str(self._runStartToStart)
-                + 's start-to-start.')
+                + ' runs per trial, spaced ' + str(self._runStartToStart)
+                + 's apart.')
             out.append(
                 '     and ' + str(self._sweepsPerRun)
-                + ' sweeps per run, measuring ' + str(self._sweepStartToStart)
-                + ' s start-to-start.')
+                + ' sweeps per run, spaced ' + str(self._sweepStartToStart)
+                + 's apart.')
         else:   # pragma: no cover
             out.append('Protocol data could not be determined.')
         out.append('Sampling rate: ' + str(self._rate) + ' Hz')
@@ -363,14 +386,16 @@ class AbfFile(object):
         if self._sweeps:
             for i, c in enumerate(self._sweeps[0]._channels):
                 out.append('Channel ' + str(i) + ': "' + c._name + '"')
-                if c._type:
+                if c._type:  # pragma: no cover
+                    # Cover pragma: Don't have appropriate test file
                     out.append('  Type: ' + type_mode_names[c._type])
                 out.append('  Unit: ' + c._unit.strip())
                 if c._lopass:
                     out.append('  Low-pass filter: ' + str(c._lopass) + ' Hz')
                 if c._cm:
                     out.append('  Cm (telegraphed): ' + str(c._cm) + ' pF')
-                if c._rs:
+                if c._rs:   # pragma: no cover
+                    # Cover pragma: Don't have appropriate test file
                     out.append('  Rs (telegraphed): ' + str(c._rs))
 
         # Methods
@@ -393,7 +418,8 @@ class AbfFile(object):
                 n = name + '[' + str(index) + ']'
                 if type(item) == OrderedDict:
                     show_dict(n, item, tab)
-                elif type(item) == list:
+                elif type(item) == list:    # pragma: no cover
+                    # Cover pragma: Don't have appropriate test file
                     show_list(n, item, tab)
                 else:
                     out.append(tab + n + ': ' + str(item))
@@ -453,15 +479,20 @@ class AbfFile(object):
                 ad_channels.append([])
             for i in range(self.protocol_channels()):
                 da_channels.append([])
+
+            # Add ad channels
             for sweep in self:
                 for channel in sweep:
                     time.append(channel.times())
                     break
                 for i, channel in enumerate(sweep):
                     ad_channels[i].append(channel.values())
+
+            # Add da channels
             for sweep in self.protocol():
                 for i, channel in enumerate(sweep):
                     da_channels[i].append(channel.values())
+
             # Combine into time series, store in log
             log['time'] = np.concatenate(time)
             log.set_time_key('time')
@@ -469,6 +500,7 @@ class AbfFile(object):
                 log['ad', i] = np.concatenate(channel)
             for i, channel in enumerate(da_channels):
                 log['da', i] = np.concatenate(channel)
+
         return log
 
     def myokit_protocol(self, channel=None, ms=True):
@@ -500,7 +532,9 @@ class AbfFile(object):
                 raise NotImplementedError('User lists are not supported.')
         else:   # pragma: no cover
             for userlist in self._header['listUserListInfo']:
-                if userlist['nULEnable']:
+                en1 = 'nULEnable' in userlist and userlist['nULEnable']
+                en2 = 'nConditEnable' in userlist and userlist['nConditEnable']
+                if en1 or en2:
                     raise NotImplementedError('User lists are not supported.')
 
         # Create protocol
@@ -666,6 +700,8 @@ class AbfFile(object):
             """
             Read and unpack a file section using the given format ``form``.
             """
+            # Form must be a str (so bytes on Python 2)
+            form = str(form)
             if offset is not None:
                 f.seek(offset)
             return struct.unpack(form, f.read(struct.calcsize(form)))
@@ -733,7 +769,8 @@ class AbfFile(object):
 
                 # Version 1: Only read tags
                 tags = []
-                for i in range(header['lNumTagEntries']):
+                for i in range(header['lNumTagEntries']):  # pragma: no cover
+                    # Cover pragma: Don't have appropriate test file
                     f.seek(header['lTagSectionPtr'] + i * 64)
                     tag = OrderedDict()
                     for key, form in TagInfoDescription:
@@ -778,7 +815,8 @@ class AbfFile(object):
                 tags = []
                 offs = sections['Tag']['index'] * BLOCKSIZE
                 size = sections['Tag']['data']
-                for i in range(sections['Tag']['length']):
+                for i in range(sections['Tag']['length']):  # pragma: no cover
+                    # Cover pragma: Don't have appropriate test file
                     f.seek(offs + i * size)
                     tag = OrderedDict()
                     for key, form in TagInfoDescription:
@@ -947,6 +985,10 @@ class AbfFile(object):
                     }
             self._epoch_functions = (dinfo, einfo_exists, einfo)
 
+        # If sweepStartToStart == 0, we set it to the duration of a sweep
+        if self._sweepStartToStart == 0:    # pragma: no cover
+            self._sweepStartToStart = nSam / self._rate
+
         # Step 2: Generate analog signals corresponding to the waveforms
         # suggested by the 'epochs' in the protocol
         # User lists are not supported
@@ -955,7 +997,9 @@ class AbfFile(object):
                 return []
         else:   # pragma: no cover
             for userlist in self._header['listUserListInfo']:
-                if userlist['nULEnable']:
+                if 'nULEnable' in userlist and userlist['nULEnable']:
+                    return []
+                if 'nConditEnable' in userlist and userlist['nConditEnable']:
                     return []
         sweeps = []
 
@@ -1065,7 +1109,8 @@ class AbfFile(object):
         if n > 0:
             dt = [(str('offset'), str('i4')), (str('len'), str('i4'))]
             sdata = np.memmap(self._filepath, dt, 'r', shape=(n), offset=o)
-        else:
+        else:   # pragma: no cover
+            # Cover pragma: Don't have appropriate test file
             sdata = np.empty((1), dt)
             sdata[0]['len'] = data.size
             sdata[0]['offset'] = 0
@@ -1136,7 +1181,8 @@ class AbfFile(object):
                         c._lopass = get('fTelegraphFilter')
 
                     # Updated low-pass cutoff
-                    if header['nSignalType'] != 0:
+                    if header['nSignalType'] != 0:  # pragma: no cover
+                        # Cover pragma: Don't have appropriate test file
                         # If a signal conditioner is used, the cutoff frequency
                         # is an undescribed "complex function" of both low-pass
                         # settings...
@@ -1161,7 +1207,8 @@ class AbfFile(object):
                             header['listADCInfo'][i]['fTelegraphFilter'])
 
                     # Updated low-pass cutoff
-                    if 'nSignalType' in header['protocol']:
+                    if 'nSignalType' in header['protocol']:  # pragma: no cover
+                        # Cover pragma: Don't have appropriate test file
                         if header['protocol']['nSignalType'] != 0:
                             # If a signal conditioner is used, the cutoff
                             # frequency is an undescribed "complex function" of
@@ -1198,7 +1245,8 @@ class AbfFile(object):
                     / h['fADCRange'])
 
                 # Signal conditioner used?
-                if h['nSignalType'] != 0:
+                if h['nSignalType'] != 0:   # pragma: no cover
+                    # Cover pragma: Don't have appropriate test file
                     f *= h['fSignalGain'][i]
 
                 # Additional gain?
@@ -1212,7 +1260,8 @@ class AbfFile(object):
                 s = h['fInstrumentOffset'][i]
 
                 # Signal conditioner used?
-                if h['nSignalType'] != 0:
+                if h['nSignalType'] != 0:   # pragma: no cover
+                    # Cover pragma: Don't have appropriate test file
                     s -= h['fSignalOffset'][i]
 
                 # Set final offset
@@ -1231,7 +1280,8 @@ class AbfFile(object):
                     / p['fADCRange'])
 
                 # Signal conditioner used?
-                if 'nSignalType' in h:
+                if 'nSignalType' in h:  # pragma: no cover
+                    # Cover pragma: Don't have appropriate test file
                     if h['nSignalType'] != 0:
                         f *= a[i]['fSignalGain']
 
@@ -1246,7 +1296,8 @@ class AbfFile(object):
                 s = a[i]['fInstrumentOffset']
 
                 # Signal conditioner used?
-                if 'nSignalType' in h:
+                if 'nSignalType' in h:  # pragma: no cover
+                    # Cover pragma: Don't have appropriate test file
                     if h['nSignalType'] != 0:
                         s -= a[i]['fSignalOffset']
 
@@ -1566,27 +1617,27 @@ TagInfoDescription = [
 
 # ABF2 Fields in the protocol section
 protocolFields = [
-    ('nOperationMode', 'h'),
-    ('fADCSequenceInterval', 'f'),
-    ('bEnableFileCompression', 'b'),
-    ('sUnused1', '3s'),
-    ('uFileCompressionRatio', 'I'),
-    ('fSynchTimeUnit', 'f'),
-    ('fSecondsPerRun', 'f'),
-    ('lNumSamplesPerEpisode', 'i'),
-    ('lPreTriggerSamples', 'i'),
-    ('lSweepsPerRun', 'i'),
-    ('lRunsPerTrial', 'i'),
-    ('lNumberOfTrials', 'i'),
-    ('nAveragingMode', 'h'),
-    ('nUndoRunCount', 'h'),
-    ('nFirstEpisodeInRun', 'h'),
-    ('fTriggerThreshold', 'f'),
-    ('nTriggerSource', 'h'),
-    ('nTriggerAction', 'h'),
-    ('nTriggerPolarity', 'h'),
-    ('fScopeOutputInterval', 'f'),
-    ('fSweepStartToStart', 'f'),
+    ('nOperationMode', 'h'),                # 0
+    ('fADCSequenceInterval', 'f'),          # 2
+    ('bEnableFileCompression', 'b'),        # 6
+    ('sUnused1', '3s'),                     # 7
+    ('uFileCompressionRatio', 'I'),         # 10
+    ('fSynchTimeUnit', 'f'),                # 14
+    ('fSecondsPerRun', 'f'),                # 18
+    ('lNumSamplesPerEpisode', 'i'),         # 22
+    ('lPreTriggerSamples', 'i'),            # 26
+    ('lSweepsPerRun', 'i'),                 # 30
+    ('lRunsPerTrial', 'i'),                 # 34
+    ('lNumberOfTrials', 'i'),               # 38
+    ('nAveragingMode', 'h'),                # 42
+    ('nUndoRunCount', 'h'),                 # 44
+    ('nFirstEpisodeInRun', 'h'),            # 46
+    ('fTriggerThreshold', 'f'),             # 48
+    ('nTriggerSource', 'h'),                # 52
+    ('nTriggerAction', 'h'),                # 54
+    ('nTriggerPolarity', 'h'),              # 56
+    ('fScopeOutputInterval', 'f'),          # 58
+    ('fSweepStartToStart', 'f'),            # 62
     ('fRunStartToStart', 'f'),
     ('lAverageCount', 'i'),
     ('fTrialStartToStart', 'f'),
