@@ -370,15 +370,6 @@ class CellMLImporter(myokit.formats.Importer):
                         + str(int2[2]) + '.')
 
                 # Now point reference at variable or reference in other comp
-                try:
-                    ref = rfs1[ref1] if ref_to_one else rfs2[ref2]
-                except KeyError:    # pragma: no cover
-                    # Cover pragma: This should already have been caught
-                    a, b = ref2, ref1 if ref_to_one else ref1, ref2
-                    self.logger().warn(
-                        'Unable to resolve reference of ' + str(a) + ' to '
-                        + str(b) + '.')
-                    continue
                 if ref_to_one:
                     rfs2[ref2] = (cname1, ref1)
                     self.logger().log(
