@@ -9,7 +9,7 @@
 from __future__ import absolute_import, division
 from __future__ import print_function, unicode_literals
 
-from ._exporter import CudaKernelExporter
+from ._exporter import CudaKernelExporter, CudaKernelRLExporter
 from ._ewriter import CudaExpressionWriter
 from myokit.formats import ansic
 
@@ -18,6 +18,7 @@ from myokit.formats import ansic
 # Exporters
 _exporters = {
     'cuda-kernel': CudaKernelExporter,
+    'cuda-kernel-rl': CudaKernelRLExporter,
 }
 
 
