@@ -23,22 +23,22 @@ import myokit
 import myokit.gui
 
 # Myokit components
-from myokit.formats.axon import AbfFile
-from myokit.formats.wcp import WcpFile
+import myokit.formats.axon
+import myokit.formats.wcp
 
 # Matplotlib (must be imported _after_ gui has had chance to set backend)
 import matplotlib
 import matplotlib.figure
 from myokit.gui import matplotlib_backend as backend
 
-# Numpy
+# NumPy
 import numpy as np
 
 # ConfigParser in Python 2 and 3
 try:
-    from ConfigParser import ConfigParser
+    import ConfigParser as configparser
 except ImportError:
-    from configparser import RawConfigParser as ConfigParser
+    import configparser
 
 
 # Application title
@@ -84,13 +84,14 @@ ABOUT = '<h1>' + TITLE + '</h1>' + """
 LICENSE = myokit.LICENSE_HTML
 
 # File filters
-FILTER_ABF = 'ABF files (*.abf)'
+FILTER_ABF = 'ABF files (*.abf *.pro)'
+FILTER_ATF = 'ATF files (*.atf)'
 FILTER_CSV = 'CSV files (*.csv)'
 FILTER_MAT = 'MAT files (*.mat)'
 FILTER_TXT = 'TXT files (*.txt)'
 FILTER_WCP = 'WCP files (*.wcp)'
 FILTER_ANY = 'All files (*.*)'
-FILTER_ALL = 'Data files (*.abf *.csv *.mat *.txt *.wcp)'
+FILTER_ALL = 'Data files (*.abf *.csv *.mat *.pro *.txt *.wcp)'
 FILTER_LIST = ';;'.join([
     FILTER_ALL,
     FILTER_ABF,
@@ -109,7 +110,7 @@ class DataLogViewer(myokit.gui.MyokitApplication):
     def __init__(self, *filenames):
         super(DataLogViewer, self).__init__()
         # Set Title, icon
-        self.setWindowTitle(TITLE + ' ' + myokit.VERSION)
+        self.setWindowTitle(TITLE + ' ' + myokit.__version__)
         # Set size, center
         self.resize(800, 600)
         qr = self.frameGeometry()
@@ -239,11 +240,12 @@ class DataLogViewer(myokit.gui.MyokitApplication):
         Loads the user configuration from an ini file.
         """
         # Read ini file
-        inifile = os.path.expanduser(SETTINGS_FILE)
-        if not os.path.isfile(inifile):
-            return
-        config = ConfigParser()
-        config.read(inifile)
+        config = configparser.RawConfigParser()
+        try:
+            config.read(os.path.expanduser(SETTINGS_FILE))
+        except configparser.ParsingError:
+            # Partially read config causes all sorts of errors, so discard
+            config = configparser.RawConfigParser()
 
         # Window dimensions and location
         if config.has_section('window'):
@@ -274,8 +276,10 @@ class DataLogViewer(myokit.gui.MyokitApplication):
         root, ext = os.path.splitext(os.path.basename(filename))
         actions = {
             '.abf': self.load_abf_file,
+            '.atf': self.load_atf_file,
             '.csv': self.load_csv_file,
             '.mat': self.load_mat_file,
+            '.pro': self.load_abf_file,
             '.txt': self.load_txt_file,
             '.wcp': self.load_wcp_file,
         }
@@ -292,7 +296,7 @@ class DataLogViewer(myokit.gui.MyokitApplication):
         Loads an abf file.
         """
         try:
-            abf = AbfFile(filename)
+            abf = myokit.formats.axon.AbfFile(filename)
         except Exception:
             e = traceback.format_exc()
             QtWidgets.QMessageBox.critical(self, TITLE, e)
@@ -300,6 +304,19 @@ class DataLogViewer(myokit.gui.MyokitApplication):
         self._path = os.path.dirname(filename)
         self._tabs.addTab(AbfTab(self, abf), os.path.basename(filename))
 
+    def load_atf_file(self, filename):
+        """
+        Loads an ATF file.
+        """
+        try:
+            atf = myokit.formats.axon.AtfFile(filename)
+        except Exception:
+            e = traceback.format_exc()
+            QtWidgets.QMessageBox.critical(self, TITLE, e)
+            return
+        self._path = os.path.dirname(filename)
+        self._tabs.addTab(AtfTab(self, atf), os.path.basename(filename))
+
     def load_csv_file(self, filename):
         """
         Loads a csv file.
@@ -352,7 +369,7 @@ class DataLogViewer(myokit.gui.MyokitApplication):
         Loads a wcp file.
         """
         try:
-            wcp = WcpFile(filename)
+            wcp = myokit.formats.wcp.WcpFile(filename)
         except Exception:
             e = traceback.format_exc()
             QtWidgets.QMessageBox.critical(self, TITLE, e)
@@ -364,7 +381,8 @@ class DataLogViewer(myokit.gui.MyokitApplication):
         """
         Saves the user configuration to an ini file.
         """
-        config = ConfigParser()
+        config = configparser.RawConfigParser()
+
         # Window dimensions and location
         config.add_section('window')
         g = self.geometry()
@@ -372,9 +390,11 @@ class DataLogViewer(myokit.gui.MyokitApplication):
         config.set('window', 'y', str(g.y()))
         config.set('window', 'w', str(g.width()))
         config.set('window', 'h', str(g.height()))
+
         # Current files, directory, etc
         config.add_section('files')
         config.set('files', 'path', self._path)
+
         # Write configuration to ini file
         inifile = os.path.expanduser(SETTINGS_FILE)
         with open(inifile, 'w') as configfile:
@@ -490,6 +510,77 @@ class AbfTab(QtWidgets.QTabWidget):
         super(AbfTab, self).deleteLater()
 
 
+class AtfTab(QtWidgets.QTabWidget):
+    """
+    A widget displaying an AGF file.
+    """
+    def __init__(self, parent, atf):
+        super(AtfTab, self).__init__(parent)
+        self._atf = atf
+
+        self.setTabsClosable(False)
+        self.setTabPosition(self.East)
+
+        self._figures = []
+        self._axes = []
+        keys = list(self._atf.keys())
+        if len(keys) > 1:
+            time = keys[0]  # Time is always first (and regularly sampled)
+            for key in keys[1:]:
+                self.addTab(self.create_graph_tab(time, key), key)
+        self.addTab(self.create_info_tab(), 'Info')
+        del(self._atf)
+
+    def create_graph_tab(self, time, key):
+        """
+        Creates a widget displaying a graph.
+        """
+        widget = QtWidgets.QWidget(self)
+
+        # Create figure
+        figure = matplotlib.figure.Figure()
+        figure.suptitle(self._atf.filename())
+        canvas = backend.FigureCanvasQTAgg(figure)
+        canvas.setParent(widget)
+        axes = figure.add_subplot(1, 1, 1)
+        toolbar = backend.NavigationToolbar2QT(canvas, widget)
+
+        # Draw lines
+        axes.plot(self._atf[time], self._atf[key])
+
+        # Create a layout
+        vbox = QtWidgets.QVBoxLayout()
+        vbox.addWidget(canvas)
+        vbox.addWidget(toolbar)
+        widget.setLayout(vbox)
+        self._figures.append(figure)
+        self._axes.append(axes)
+
+        # Return widget
+        return widget
+
+    def create_info_tab(self):
+        """
+        Creates a tab displaying information about the file.
+        """
+        widget = QtWidgets.QTextEdit(self)
+        widget.setText(self._atf.info())
+        widget.setReadOnly(True)
+        return widget
+
+    def deleteLater(self):
+        """
+        Deletes this tab (later).
+        """
+        for figure in self._figures:
+            figure.clear()
+        for axes in self._axes:
+            axes.cla()
+        del(self._figures, self._axes)
+        gc.collect()
+        super(AtfTab, self).deleteLater()
+
+
 class CsvTab(QtWidgets.QTabWidget):
     """
     A widget displaying a CSV file.
