@@ -39,9 +39,9 @@ import matplotlib.pyplot as plt     # noqa -- Make flake8 ignore this line
 
 # ConfigParser in Python 2 and 3
 try:
-    from ConfigParser import ConfigParser
+    import ConfigParser as configparser
 except ImportError:
-    from configparser import RawConfigParser as ConfigParser
+    import configparser
 
 
 # Application title
@@ -178,6 +178,9 @@ class MyokitIDE(myokit.gui.MyokitApplication):
         self._model_editor.redoAvailable.connect(self.change_redo_model)
         self._protocol_editor.redoAvailable.connect(self.change_redo_protocol)
         self._script_editor.redoAvailable.connect(self.change_redo_script)
+        self._model_editor.copyAvailable.connect(self.change_copy_model)
+        self._protocol_editor.copyAvailable.connect(self.change_copy_protocol)
+        self._script_editor.copyAvailable.connect(self.change_copy_script)
 
         # Create console
         self._console = Console()
@@ -372,6 +375,18 @@ class MyokitIDE(myokit.gui.MyokitApplication):
         except Exception:
             self.show_exception()
 
+    def action_copy(self):
+        """
+        Copy text in editor (when triggered from menu).
+        """
+        self._editor_tabs.currentWidget().copy()
+
+    def action_cut(self):
+        """
+        Cut text in editor (when triggered from menu).
+        """
+        self._editor_tabs.currentWidget().cut()
+
     def action_explore(self):
         """
         Opens the explorer
@@ -509,6 +524,23 @@ class MyokitIDE(myokit.gui.MyokitApplication):
                 editor.hide_find_dialog()
         current.activate_find_dialog()
 
+    def action_format_protocol(self):
+        """
+        Reformat the protocol.
+        """
+        try:
+            before = self._protocol_editor.get_text()
+            p = self.protocol(errors_in_console=True)
+            if p is False:
+                self._console.write(
+                    'Unable to apply formatting: Errors in protocol.')
+            elif p is not None:
+                after = p.code()
+                if after != before:
+                    self._protocol_editor.replace(after)
+        except Exception:
+            self.show_exception()
+
     def action_import_abf_protocol(self):
         """
         Imports a protocol from an abf (v2) file.
@@ -717,6 +749,12 @@ class MyokitIDE(myokit.gui.MyokitApplication):
             self.statusBar().showMessage('"Open recent file" failed')
             self.show_exception()
 
+    def action_paste(self):
+        """
+        Paste text in editor (when triggered from menu).
+        """
+        self._editor_tabs.currentWidget().paste()
+
     def action_preview_protocol(self):
         """
         Displays a preview of the current protocol.
@@ -1123,18 +1161,50 @@ class MyokitIDE(myokit.gui.MyokitApplication):
         self._recent_files = self._recent_files[:N_RECENT_FILES]
         self.update_recent_files_menu()
 
+    def change_copy_model(self, enabled):
+        """
+        Qt slot: CopyAvailable state of model editor changed.
+        """
+        if self._editor_tabs.currentWidget() == self._model_editor:
+            self._tool_copy.setEnabled(enabled)
+            self._tool_cut.setEnabled(enabled)
+
+    def change_copy_protocol(self, enabled):
+        """
+        Qt slot: CopyAvailable state of protocol editor changed.
+        """
+        if self._editor_tabs.currentWidget() == self._protocol_editor:
+            self._tool_copy.setEnabled(enabled)
+            self._tool_cut.setEnabled(enabled)
+
+    def change_copy_script(self, enabled):
+        """
+        Qt slot: CopyAvailable state of script editor changed.
+        """
+        if self._editor_tabs.currentWidget() == self._script_editor:
+            self._tool_copy.setEnabled(enabled)
+            self._tool_cut.setEnabled(enabled)
+
     def change_editor_tab(self, index):
         """
         Qt slot: Called when the editor tab is changed.
         """
+        # Update copy/cut
+        w = self._editor_tabs.currentWidget()
+        e = w.textCursor().hasSelection()
+        self._tool_copy.setEnabled(e)
+        self._tool_cut.setEnabled(e)
+
         # Update undo/redo
-        d = self._editor_tabs.currentWidget().document()
+        d = w.document()
         self._tool_undo.setEnabled(d.isUndoAvailable())
         self._tool_redo.setEnabled(d.isRedoAvailable())
+
         # Hide find/replace dialogs
         es = (self._model_editor, self._protocol_editor, self._script_editor)
         for editor in es:
             editor.hide_find_dialog()
+
         # Show/hide model navigator
         if index == 0:
             self._tool_view_navigator.setEnabled(True)
@@ -1144,6 +1214,12 @@ class MyokitIDE(myokit.gui.MyokitApplication):
             self._tool_view_navigator.setEnabled(False)
             self._navigator.hide()
 
+        # Enabled/disable protocol tools
+        if index == 1:
+            self._tool_format_protocol.setEnabled(True)
+        else:
+            self._tool_format_protocol.setEnabled(False)
+
     def change_model(self):
         """
         Qt slot: Called whenever the model is changed.
@@ -1358,6 +1434,34 @@ class MyokitIDE(myokit.gui.MyokitApplication):
         self._menu_edit.addAction(self._tool_redo)
         # Edit > ----
         self._menu_edit.addSeparator()
+        # Edit > Cut
+        self._tool_cut = QtWidgets.QAction('&Cut', self)
+        self._tool_cut.setShortcut('Ctrl+X')
+        self._tool_cut.setStatusTip(
+            'Cut the selected text and copy it to the clipboard.')
+        #self._tool_cut.setIcon(myokit.gui.icon('edit-cut'))
+        self._tool_cut.triggered.connect(self.action_cut)
+        self._tool_cut.setEnabled(False)
+        self._menu_edit.addAction(self._tool_cut)
+        # Edit > Copy
+        self._tool_copy = QtWidgets.QAction('&Copy', self)
+        self._tool_copy.setShortcut('Ctrl+C')
+        self._tool_copy.setStatusTip(
+            'Copy the selected text to the clipboard.')
+        #self._tool_copy.setIcon(myokit.gui.icon('edit-copy'))
+        self._tool_copy.triggered.connect(self.action_copy)
+        self._tool_copy.setEnabled(False)
+        self._menu_edit.addAction(self._tool_copy)
+        # Edit > Paste
+        self._tool_paste = QtWidgets.QAction('&Paste', self)
+        self._tool_paste.setShortcut('Ctrl+V')
+        self._tool_paste.setStatusTip(
+            'Paste text from the clipboard into the editor.')
+        #self._tool_paste.setIcon(myokit.gui.icon('edit-paste'))
+        self._tool_paste.triggered.connect(self.action_paste)
+        self._menu_edit.addAction(self._tool_paste)
+        # Edit > ----
+        self._menu_edit.addSeparator()
         # Edit > Find and replace
         self._tool_find = QtWidgets.QAction('&Find and replace', self)
         self._tool_find.setShortcut('Ctrl+F')
@@ -1367,6 +1471,17 @@ class MyokitIDE(myokit.gui.MyokitApplication):
         self._menu_edit.addAction(self._tool_find)
         # Edit > ----
         self._menu_edit.addSeparator()
+        # Edit > Format protocol
+        self._tool_format_protocol = QtWidgets.QAction(
+            'Format protocol', self)
+        self._tool_format_protocol.setStatusTip(
+            'Standardise the formatting of the protocol section.')
+        self._tool_format_protocol.setEnabled(False)
+        self._tool_format_protocol.triggered.connect(
+            self.action_format_protocol)
+        self._menu_edit.addAction(self._tool_format_protocol)
+        # Edit > ----
+        self._menu_edit.addSeparator()
         # Edit > Comment or uncomment
         self._tool_comment = QtWidgets.QAction(
             '&Comment/uncomment selected lines', self)
@@ -1716,11 +1831,12 @@ class MyokitIDE(myokit.gui.MyokitApplication):
         Loads the user configuration from an ini file.
         """
         # Read ini file
-        inifile = os.path.expanduser(SETTINGS_FILE)
-        if not os.path.isfile(inifile):
-            return
-        config = ConfigParser()
-        config.read(inifile)
+        config = configparser.RawConfigParser()
+        try:
+            config.read(os.path.expanduser(SETTINGS_FILE))
+        except configparser.ParsingError:
+            # Partially read config causes all sorts of errors, so discard
+            config = configparser.RawConfigParser()
 
         def getor(section, name, alt):
             """ Get or use alternative """
@@ -1778,36 +1894,48 @@ class MyokitIDE(myokit.gui.MyokitApplication):
         """
         # Close explorer, if required
         self.close_explorer()
+
         # Allow user directory and relative paths
         filename = os.path.abspath(os.path.expanduser(filename))
+
         # Set path to filename's path. Do this before we even know the file is
         # valid: if you click the wrong file by mistake you shouldn't have to
         # browse all the way back again).
         self._path = os.path.dirname(filename)
+
         # Open file, split into segments
         with open(filename, 'r') as f:
             segments = myokit.split(f)
+
         # Still here? Then set as file.
         self._file = filename
+
         # Add to recent files
         self.add_recent_file(filename)
+
         # Update model editor
         self._model_editor.set_text(segments[0].strip())
+
         # Update protocol editor
         self._protocol_editor.set_text(segments[1].strip())
+
         # Update script editor
         self._script_editor.set_text(segments[2].strip())
+
         # Don't validate model or protocol. Opening an invalid file is not an
         # error in itself.
         # Update console
         self._console.write('Opened ' + self._file)
+
         # Set working directory to file's path
         os.chdir(self._path)
+
         # Save settings file
         try:
             self.save_config()
         except Exception:
             pass
+
         # For some reason, setPlainText('') triggers a change event claiming
         # the text has changed. As a result, files with empty sections will
         # always show up as changed. This is prevented manually below:
@@ -1815,6 +1943,7 @@ class MyokitIDE(myokit.gui.MyokitApplication):
         self._model_editor.document().setModified(False)
         self._protocol_editor.document().setModified(False)
         self._script_editor.document().setModified(False)
+
         # Update interface
         self.update_navigator()     # Avoid delay
         self.update_window_title()
@@ -1960,14 +2089,17 @@ class MyokitIDE(myokit.gui.MyokitApplication):
             if console:
                 self._console.write('No changes to protocol since last build.')
             return self._valid_protocol
+
         # Parse and validate
         protocol = None
+
         # Check for empty protocol field
         lines = self._protocol_editor.get_text()
         if lines.strip() == '':
             if console:
                 self._console.write('No protocol found.')
             return None
+
         # Validate and return
         lines = lines.splitlines()
         try:
@@ -1988,7 +2120,8 @@ class MyokitIDE(myokit.gui.MyokitApplication):
         """
         Saves the user configuration to an ini file.
         """
-        config = ConfigParser()
+        config = configparser.RawConfigParser()
+
         # Window dimensions and location
         config.add_section('window')
         g = self.geometry()
@@ -1996,25 +2129,30 @@ class MyokitIDE(myokit.gui.MyokitApplication):
         config.set('window', 'y', str(g.y()))
         config.set('window', 'w', str(g.width()))
         config.set('window', 'h', str(g.height()))
+
         # Central splitter
         config.add_section('splitter')
         a, b = self._central_splitter.sizes()
         config.set('splitter', 'top', str(a))
         config.set('splitter', 'bottom', str(b))
+
         # Current and recent files
         config.add_section('files')
         config.set('files', 'file', self._file)
         for k, filename in enumerate(self._recent_files):
             config.set('files', 'recent_' + str(k), filename)
+
         # Source editors
         self._model_editor.save_config(config, 'model_editor')
         self._protocol_editor.save_config(config, 'protocol_editor')
         self._script_editor.save_config(config, 'script_editor')
+
         # Model navigator visibility
         config.add_section('model_navigator')
         config.set(
             'model_navigator', 'visible',
             str(self._tool_view_navigator.isChecked()))
+
         # Write configuration to ini file
         inifile = os.path.expanduser(SETTINGS_FILE)
         with open(inifile, 'w') as configfile:
@@ -2171,7 +2309,7 @@ class MyokitIDE(myokit.gui.MyokitApplication):
         """
         Sets this window's title based on the current state.
         """
-        title = TITLE + ' ' + myokit.VERSION
+        title = TITLE + ' ' + myokit.__version__
         if self._file:
             title = os.path.basename(self._file) + ' - ' + title
             if self._have_changes:
@@ -2181,10 +2319,10 @@ class MyokitIDE(myokit.gui.MyokitApplication):
 
 class Console(QtWidgets.QPlainTextEdit):
     """
-    *Extends*: ``QtWidgets.QPlainTextEdit``
-
     Console window used to write plain text output to in the IDE. Shows model
     parsing states and output of running explorations / scripts.
+
+    *Extends*: ``QtWidgets.QPlainTextEdit``
     """
     def __init__(self, parent=None):
         super(Console, self).__init__(parent)
