@@ -39,9 +39,15 @@ try:
 except ImportError:  # pragma: no python 3 cover
     import Queue as queue
 
+# String types in Python 2 and 3
+try:
+    basestring
+except NameError:   # pragma: no python 2 cover
+    basestring = str
+
 
 #
-# This module is deprecated!
+# Deprecated since 2018-04-16
 #
 import logging
 logger = logging.getLogger('myokit')
@@ -240,7 +246,7 @@ def cmaes(
     # Check hint
     if hint is None:
         hint = lower + 0.5 * (upper - lower)
-    elif hint == 'random':
+    elif isinstance(hint, basestring) and hint == 'random':
         hint = lower + np.random.uniform(0, 1, d) * (upper - lower)
     else:
         hint = np.array(hint, copy=True)
@@ -324,7 +330,7 @@ def cmaes(
     # If no seed is given, CMAES reseeds. So to get consistent results always
     # pass in a seed. This can be random generated (because then you can still
     # seed np.random to get consistent output).
-    options.set('seed', np.random.randint(2**32))
+    options.set('seed', np.random.randint(2**31))
 
     # Start one or multiple runs
     for i in range(1 + ipop):
@@ -344,7 +350,7 @@ def cmaes(
             candidates = es.ask()
             es.tell(candidates, evaluator.evaluate(candidates))
             if callback is not None:
-                r = es.result()
+                r = es.result
                 callback(np.array(r[0], copy=True), r[1])
             if verbose:
                 es.disp()
@@ -752,8 +758,6 @@ class _ParameterTransformWrapper(object):
 
 class ParallelEvaluator(Evaluator):
     """
-    *Extends:* :class:`Evaluator`
-
     Evaluates a single-valued function object for any set of input values
     given, using all available cores.
 
@@ -803,6 +807,8 @@ multiprocessing.html#all-platforms>`_ for details).
     Note that while this class uses multiprocessing, it is not thread/process
     safe itself: It should not be used by more than a single thread/process at
     a time.
+
+    *Extends:* :class:`Evaluator`
     """
     def __init__(
             self, function, nworkers=None, max_tasks_per_worker=500,
@@ -1414,8 +1420,6 @@ def pso(
 
 class SequentialEvaluator(Evaluator):
     """
-    *Extends:* :class:`Evaluator`
-
     Evaluates a function (or callable object) for a list of input values.
 
     Runs sequentially, but shares an interface with the
@@ -1430,6 +1434,8 @@ class SequentialEvaluator(Evaluator):
         specified, ``f`` will be called as ``f(x, *args)``.
 
     Returns a list containing the calculated function evaluations.
+
+    *Extends:* :class:`Evaluator`
     """
     def __init__(self, function, args=None):
         super(SequentialEvaluator, self).__init__(function, args)
@@ -1549,7 +1555,7 @@ def snes(
     # Check hint
     if hint is None:
         hint = lower + 0.5 * (upper - lower)
-    elif hint == 'random':
+    elif isinstance(hint, basestring) and hint == 'random':
         hint = lower + np.random.uniform(0, 1, d) * (upper - lower)
     else:
         hint = np.array(hint, copy=True)
@@ -1982,8 +1988,6 @@ def voronoi_regions(x, y, f, xlim, ylim):
 #
 class _Worker(multiprocessing.Process):
     """
-    *Extends:* ``multiprocessing.Process``
-
     Worker class for use with :class:`ParallelEvaluator`.
 
     Evaluates a single-valued function for every point in a ``tasks`` queue
@@ -2015,6 +2019,7 @@ class _Worker(multiprocessing.Process):
         This flag will be set by the worker whenever it encounters an
         error.
 
+    *Extends:* ``multiprocessing.Process``
     """
 
     def __init__(
@@ -2168,7 +2173,7 @@ def xnes(
     # Check hint
     if hint is None:
         hint = lower + 0.5 * (upper - lower)
-    elif hint == 'random':
+    elif isinstance(hint, basestring) and hint == 'random':
         hint = lower + np.random.uniform(0, 1, d) * (upper - lower)
     else:
         hint = np.array(hint, copy=True)
