@@ -1,5 +1,5 @@
 #
-# Tools for working with Markov models
+# Tools for working with Markov ion channel models
 #
 # This file is part of Myokit
 #  Copyright 2011-2018 Maastricht University, University of Oxford
@@ -10,7 +10,9 @@ from __future__ import absolute_import, division
 from __future__ import print_function, unicode_literals
 
 import numpy as np
+
 import myokit
+from myokit.formats import sympy
 
 
 class LinearModel(object):
@@ -28,7 +30,11 @@ class LinearModel(object):
     derivative ``dot(x)`` and a current ``I``.
 
     ``A`` and ``B`` can contain non-linear functions, but should be simple
-    scalar matrices when evaluated for a fixed ``V`` and ``p``.
+    scalar matrices when evaluated for a fixed ``V`` and ``p``. For example,
+    the current equation ``I = g * O * (V - E)`` would have ``p = [g, E]``, so
+    that for fixed ``p`` and ``V`` this would resolve to
+    ``I = (g * (V - E)) * O``, such that ``g * (V - E)`` is a constant that can
+    be included in ``B``.
 
     The model variables to treat as parameter are specified by the user when
     the model is created. Any other variables, for example state variables such
@@ -107,7 +113,6 @@ class LinearModel(object):
         (default) the method will search for a variable with the label
         ``membrane_potential``.
 
-    *Note: this class requires Sympy to be installed.*
     """
     # NOTE: A LinearModel must be immutable!
     # This ensures that the simulations don't have to clone it (which would be
@@ -118,10 +123,6 @@ class LinearModel(object):
     def __init__(self, model, states, parameters=None, current=None, vm=None):
         super(LinearModel, self).__init__()
 
-        # Import sympy, or raise an ImportError if we can't.
-        from myokit.formats import sympy
-        del(sympy)
-
         #
         # Check input
         #
@@ -287,28 +288,29 @@ class LinearModel(object):
         (This method is called only once, by the constructor, but it's
         complicated enough to warrant its own method...)
         """
-        from myokit.formats import sympy
-
         # Create mapping from states to index
         state_indices = {}
         for k, state in enumerate(self._states):
             state_indices[state] = k
 
         # Extract expressions for state & current variables:
-        #  1. Get expanded equation
+        #  1. Get expanded equation (with all references to variables replaced
+        #     by inlined expressions)
         #  2. Convert to sympy
         #  3. Simplify / attempt to rewrite as linear combination
-        #  4. Import from sympy
+        #  4. Re-import from sympy
         expressions = []
         for state in self._states:
             e = state.rhs().clone(expand=True, retain=self._inputs)
             e = sympy.write(e)
-            e = sympy.read(e.expand(), self._model)
+            e = e.expand()
+            e = sympy.read(e, self._model)
             expressions.append(e)
         if self._current is not None:
             e = self._current.rhs().clone(expand=True, retain=self._inputs)
             e = sympy.write(e)
-            current_expression = sympy.read(e.expand(), self._model)
+            e = e.expand()
+            current_expression = sympy.read(e, self._model)
 
         # Create parametrisable matrices to evaluate the state & current
         #  1. For each expression, get the terms. This works because Sympy
@@ -417,7 +419,7 @@ class LinearModel(object):
         globl = {'numpy': np, 'n': n}
         local = {}
 
-        exec(code, globl, local)
+        myokit._exec(code, globl, local)
         self._matrix_function = local['matrix_function']
 
         #
@@ -436,7 +438,7 @@ class LinearModel(object):
         code = head + '\n' + '\n'.join(['    ' + line for line in body])
         globl = {'numpy': np}
         local = {}
-        exec(code, globl, local)
+        myokit._exec(code, globl, local)
         self._rate_list_function = local['rate_list_function']
 
     def current(self):
@@ -512,7 +514,7 @@ class LinearModel(object):
             parameters = [
                 x for x in component.variables(const=True) if x.is_literal()]
             # Sort by qname, using natural sort
-            parameters.sort(key=lambda x: myokit.natural_sort_key(x.qname()))
+            parameters.sort(key=lambda x: myokit._natural_sort_key(x.qname()))
 
         # Get current
         if current is None:
@@ -731,12 +733,6 @@ class AnalyticalSimulation(object):
         # Time variable
         self._time = 0
 
-        # If protocol was given, create pacing system, update vm
-        self._pacing = None
-        if self._protocol:
-            self._pacing = myokit.PacingSystem(self._protocol)
-            self._membrane_potential = self._pacing.advance(self._time)
-
         # Check if we have a current variable
         self._has_current = self._model.current() is not None
 
@@ -754,12 +750,30 @@ class AnalyticalSimulation(object):
         self._parameters = np.array(
             self._model.default_parameters(), copy=True, dtype=float)
 
+        # Mapping from parameter names to index in parameter array
+        self._parameter_map = {}
+        for i, p in enumerate(self._model.parameters()):
+            self._parameter_map[p] = i
+
         # Cached matrices
         self._cached_matrices = None
 
         # Cached partial solution (eigenvalue decomposition etc.)
         self._cached_solution = None
 
+        # If protocol was given, create pacing system, update vm
+        self._pacing = None
+        if self._protocol:
+            self._pacing = myokit.PacingSystem(self._protocol)
+            self._membrane_potential = self._pacing.advance(self._time)
+
+        # Keys for logging
+        self._time_key = self._model._model.time().qname()
+        self._vm_key = self._model._membrane_potential
+        self._log_keys = [self._time_key, self._vm_key] + self._model.states()
+        if self._has_current:
+            self._log_keys.append(self._model.current())
+
     def current(self, state):
         """
         Calculates the current for a given state.
@@ -846,7 +860,7 @@ class AnalyticalSimulation(object):
             self._pacing = myokit.PacingSystem(self._protocol)
             self._membrane_potential = self._pacing.advance(self._time)
 
-    def run(self, duration, log=None, log_interval=0.01):
+    def run(self, duration, log=None, log_interval=0.01, log_times=None):
         """
         Runs a simulation for ``duration`` time units.
 
@@ -864,6 +878,9 @@ class AnalyticalSimulation(object):
             results will be appended to this one.
         ``log_interval``
             The time between logged points.
+        ``log_times``
+            A pre-defined sequence of times to log at. If set, ``log_interval``
+            will be ignored.
 
         Returns a :class:`myokit.DataLog` with the simulation results.
         """
@@ -875,35 +892,31 @@ class AnalyticalSimulation(object):
         if log_interval <= 0:
             raise ValueError('Log interval must be greater than zero.')
 
+        # Check log_times
+        if log_times is None:
+            log_times = self._time + np.arange(0, duration, log_interval)
+
         # Set up logging
-        vm_key = self._model._membrane_potential
-        time_key = self._model._model.time().qname()
         if log is None:
 
             # Create new log
             log = myokit.DataLog()
-            log[time_key] = []
-            log.set_time_key(time_key)
-            log[vm_key] = []
-            for key in self._model.states():
-                log[key] = []
-            if self._has_current:
-                log[self._model.current()] = []
+            log.set_time_key(self._time_key)
+            for key in self._log_keys:
+                log[key] = np.zeros(log_times.shape)
+            offset = 0
 
         else:
 
             # Check existing log
-            n = 2 + len(self._state) + (1 if self._has_current else 0)
-            if len(log.keys()) > n:
+            if len(log.keys()) > len(self._log_keys):
                 raise ValueError('Invalid log: contains extra keys.')
-            test = self._model.states()
-            test.append(time_key)
-            test.append(vm_key)
-            if self._has_current:
-                test.append(self._model.current())
             try:
-                for key in test:
-                    log[key]
+                key = self._time_key    # Note: error msg below uses `key`
+                offset = len(log[key])
+                for key in self._log_keys:
+                    log[key] = np.concatenate((
+                        log[key], np.zeros(log_times.shape)))
             except KeyError:
                 raise ValueError(
                     'Invalid log: missing entry for <' + str(key) + '>.')
@@ -912,7 +925,7 @@ class AnalyticalSimulation(object):
         if self._protocol is None:
 
             # User defined membrane potential
-            self._run(duration, log, log_interval)
+            self._run(log, log_times, self._time + duration, offset)
 
         else:
 
@@ -921,43 +934,58 @@ class AnalyticalSimulation(object):
             while self._time < tfinal:
                 # Run simulation
                 tnext = min(tfinal, self._pacing.next_time())
-                self._run(tnext - self._time, log, log_interval)
+                times = log_times[np.logical_and(
+                    log_times >= self._time, log_times < tnext)]
+                self._run(log, times, tnext, offset)
+                offset += len(times)
+
                 # Update pacing
-                self._membrane_potential = self._pacing.advance(tnext, tfinal)
+                self._membrane_potential = self._pacing.advance(tnext)
                 self._cached_matrices = None
                 self._cached_solution = None
 
         # Return
         return log
 
-    def _run(self, duration, log, log_interval):
+    def _run(self, log, times, tnext, offset):
         """
         Runs a simulation with the current membrane potential.
+
+        Arguments:
+
+        ``log``
+            The log to append to.
+        ``times``
+            The times to evaluate at.
+        ``tnext``
+            The final time to move to.
+        ``offset``
+            The offset in the ``times`` array to log in
+
         """
         # Simulate with fixed V
-        times = np.arange(0, duration, log_interval)
         if self._has_current:
-            states, currents = self.solve(times)
+            states, currents = self.solve(times - self._time)
         else:
-            states = self.solve(times)
-        times += self._time
+            states = self.solve(times - self._time)
 
         # Log results
+        lo = offset
+        hi = lo + len(times)
         key = log.time_key()
-        log[key] = np.concatenate((log[key], times))
+        log[key][lo:hi] = times
         for i, key in enumerate(self._model.states()):
-            log[key] = np.concatenate((log[key], states[i]))
+            log[key][lo:hi] = states[i]
         if self._has_current:
             key = self._model.current()
-            log[key] = np.concatenate((log[key], currents))
-        vm_key = self._model._membrane_potential
-        log[vm_key] = np.concatenate(
-            (log[vm_key], [self._membrane_potential] * len(times)))
+            log[key][lo:hi] = currents
+        key = self._model._membrane_potential
+        log[key][lo:hi] = self._membrane_potential
 
         # Now run simulation for final time (which might not be included in the
         # list of logged times, and should not, if you want to be able to
         # append logs without creating duplicate points).
-        times = np.array([duration])
+        times = np.array([tnext - self._time])
         if self._has_current:
             states, currents = self.solve(times)
         else:
@@ -965,7 +993,13 @@ class AnalyticalSimulation(object):
 
         # Update simulation state
         self._state = np.array(states[:, -1], copy=True)
-        self._time += duration
+        self._time = tnext
+
+    def set_constant(self, variable, value):
+        """
+        Updates a single parameter to a new value.
+        """
+        self._parameters[self._parameter_map[variable]] = float(value)
 
     def set_default_state(self, state):
         """
@@ -1172,8 +1206,14 @@ class DiscreteSimulation(object):
         self._parameters = np.array(
             self._model.default_parameters(), copy=True, dtype=float)
 
-        # Cached transition rate list
+        # Mapping from parameter names to index in parameter array
+        self._parameter_map = {}
+        for i, p in enumerate(self._model.parameters()):
+            self._parameter_map[p] = i
+
+        # Cached transition rate list & current matrix
         self._cached_rates = None
+        self._cached_matrix = None
 
         # Set simulation time
         self._time = 0
@@ -1267,6 +1307,15 @@ class DiscreteSimulation(object):
             self._pacing = myokit.PacingSystem(self._protocol)
             self._membrane_potential = self._pacing.advance(self._time)
 
+    def _current_matrix(self):
+        """
+        Returns the (cached or regenerated) current matrix B.
+        """
+        if self._cached_matrix is None:
+            self._cached_matrix = self._model.matrices(
+                self._membrane_potential, self._parameters)[1]
+        return self._cached_matrix
+
     def _rates(self):
         """
         Returns the (cached or regenerated) transition rate list.
@@ -1316,23 +1365,26 @@ class DiscreteSimulation(object):
 
         # Set up logging
         time_key = self._model._model.time().qname()
-        vm_key = self._model._membrane_potential
+        log_vars = [time_key, self._model._membrane_potential]
+        log_vars.extend(self._model.states())
+        cur_key = self._model.current()
+        if cur_key is not None:
+            log_vars.append(cur_key)
+
         if log is None:
             # Create new log
             log = myokit.DataLog()
-            log[time_key] = []
             log.set_time_key(time_key)
-            log[vm_key] = []
-            for key in self._model.states():
-                log[key] = []
+            for var in log_vars:
+                log[var] = []
 
         else:
 
             # Check existing log
-            if len(log.keys()) > 2 + len(self._state):
+            if len(log.keys()) > len(log_vars):
                 raise ValueError('Invalid log: contains extra keys.')
             try:
-                for key in [vm_key, time_key] + self._model.states():
+                for key in log_vars:
                     log[key]
             except KeyError:
                 raise ValueError(
@@ -1351,8 +1403,9 @@ class DiscreteSimulation(object):
                 tnext = min(tfinal, self._pacing.next_time())
                 self._run(tnext - self._time, log)
                 # Update pacing
-                self._membrane_potential = self._pacing.advance(tnext, tfinal)
+                self._membrane_potential = self._pacing.advance(tnext)
                 self._cached_rates = None
+                self._cached_matrix = None
 
         # Return
         return log
@@ -1367,7 +1420,7 @@ class DiscreteSimulation(object):
         for key in self._model.states():
             log_states.append(log[key])
 
-        # Get current time and state
+        # Get current, time and state
         t = self._time
         state = np.array(self._state, copy=True, dtype=int)
 
@@ -1443,10 +1496,26 @@ class DiscreteSimulation(object):
         vm_key = self._model._membrane_potential
         log[vm_key].extend([self._membrane_potential] * n_steps)
 
+        # Add current to log
+        c = self._model.current()
+        if c is not None:
+            B = self._current_matrix()
+            x = np.vstack([
+                np.array(log_states[i][-n_steps:], dtype=float)
+                for i in range(len(state))])
+            x /= self._nchannels
+            log[c].extend(B.dot(x))
+
         # Update current state and time
         self._state = list(state)
         self._time += duration
 
+    def set_constant(self, variable, value):
+        """
+        Updates a single parameter to a new value.
+        """
+        self._parameters[self._parameter_map[variable]] = float(value)
+
     def set_default_state(self, state):
         """
         Changes the default state used in the simulation.
@@ -1475,6 +1544,7 @@ class DiscreteSimulation(object):
                 'Membrane potential cannot be set if a protocol is used.')
         self._membrane_potential = float(v)
         self._cached_rates = None
+        self._cached_matrix = None
 
     def set_parameters(self, parameters):
         """
@@ -1486,6 +1556,7 @@ class DiscreteSimulation(object):
                 + str(len(self._parameters)) + ') values.')
         self._parameters = np.array(parameters, copy=True, dtype=float)
         self._cached_rates = None
+        self._cached_matrix = None
 
     def set_state(self, state):
         """
@@ -1583,11 +1654,12 @@ def _find_factor(expression, original):
 
 class MarkovModel(object):
     """
-    **Deprecated**: Since version 1.22.0 this class has been replaced by the
-    classes :class:`LinearModel` and :class:`AnalyticalSimulation`. Please
-    update your code to use these classes instead. This class will be removed
-    in future versions of Myokit.
+    **Deprecated**: This class has been replaced by the classes
+    :class:`LinearModel` and :class:`AnalyticalSimulation`. Please update your
+    code to use these classes instead. This class will be removed in
+    future versions of Myokit.
     """
+
     @staticmethod
     def from_component(
             component, states=None, parameters=None, current=None, vm=None):
@@ -1595,10 +1667,24 @@ class MarkovModel(object):
         Creates and returns an :class:`AnalyticalSimulation` using a
         :class:`LinearModel` based on a Myokit model component.
         """
+        # Deprecated since 2016-01-25
+        import logging
+        logging.basicConfig()
+        log = logging.getLogger(__name__)
+        log.warning(
+            'The method `MarkovModel.from_component` is deprecated.'
+            ' Please use `LinearModel.from_component` instead.')
         return AnalyticalSimulation(LinearModel.from_component(
             component, states, parameters, current, vm))
 
     def __new__(self, model, states, parameters=None, current=None, vm=None):
+        # Deprecated since 2016-01-25
+        import logging
+        logging.basicConfig()
+        log = logging.getLogger(__name__)
+        log.warning(
+            'The `MarkovModel` class is deprecated.'
+            ' Please use the `LinearModel` class instead.')
         return AnalyticalSimulation(LinearModel(
             model, states, parameters, current, vm))
 
